//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// ChatWnd.cpp : implementation file
//
#include "stdafx.h"
#include "emule.h"
#include "ChatWnd.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// CChatWnd dialog

IMPLEMENT_DYNAMIC(CChatWnd, CDialog)
CChatWnd::CChatWnd(CWnd* pParent /*=NULL*/)
	: CResizableDialog(CChatWnd::IDD, pParent)
{
}

CChatWnd::~CChatWnd()
{
}

void CChatWnd::DoDataExchange(CDataExchange* pDX)
{
	CResizableDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_CHATSEL, m_ctlChatSelector);
	DDX_Control(pDX, IDC_LIST2, m_FriendListCtrl);
	DDX_Control(pDX, IDC_CMESSAGE, m_ctlInputText);
	DDX_Control(pDX, IDC_CSEND, m_ctlSendButton);
	DDX_Control(pDX, IDC_CCLOSE, m_ctlCloseButton);
}

BEGIN_MESSAGE_MAP(CChatWnd, CResizableDialog)
	ON_WM_KEYDOWN()
	ON_WM_SHOWWINDOW()
	ON_MESSAGE(WM_CLOSETAB, OnCloseTab)
	ON_BN_CLICKED(IDC_CCLOSE, OnBnClickedCclose)
	ON_BN_CLICKED(IDC_CSEND, OnBnClickedCsend)
	ON_WM_DESTROY()
END_MESSAGE_MAP()
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CChatWnd message handlers
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CChatWnd::OnInitDialog()
{
	CResizableDialog::OnInitDialog();

	oldMessageIcon = GetMessageStatic()->SetIcon(GetMessageIcon());
	oldFriendsIcon = GetFriendsStatic()->SetIcon(GetFriendsIcon());

	AddAnchor(IDC_CHATSEL,TOP_LEFT,BOTTOM_RIGHT);
	AddAnchor(IDC_LIST2,TOP_LEFT,BOTTOM_LEFT);
	AddAnchor(IDC_CSEND, BOTTOM_RIGHT);
	AddAnchor(IDC_CCLOSE, BOTTOM_RIGHT);
	AddAnchor(IDC_CMESSAGE, BOTTOM_LEFT, BOTTOM_RIGHT);
	AddAnchor(IDC_FRIENDS_LBL,TOP_LEFT);
	AddAnchor(IDC_FRIENDSICON,TOP_LEFT);

	m_ctlChatSelector.Init();
	m_FriendListCtrl.Init();

	Localize();

	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatWnd::StartSession(CUpDownClient* client)
{
	if (!client->GetUserName())
		return;

	g_eMuleApp.m_pdlgEmule->SetActiveDialog(this);
	m_ctlChatSelector.StartSession(client);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatWnd::OnShowWindow(BOOL bShow,UINT nStatus)
{
	if (bShow)
		m_ctlChatSelector.ShowChat();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatWnd::OnBnClickedCclose()
{
	m_ctlChatSelector.EndSession();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatWnd::OnBnClickedCsend()
{
	CString			strNewMessage;

	m_ctlInputText.GetWindowText(strNewMessage);

	strNewMessage.Trim();
	if (!strNewMessage.IsEmpty() && m_ctlChatSelector.SendMessage(strNewMessage))
		m_ctlInputText.SetWindowText(_T(""));
	m_ctlInputText.SetFocus();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CChatWnd::PreTranslateMessage(MSG* pMsg)
{
	EMULE_TRY

	if (pMsg->message == WM_KEYDOWN && (pMsg->hwnd == m_ctlInputText.m_hWnd))
	{
		if (pMsg->wParam == VK_RETURN)
		{
			OnBnClickedCsend();
			return TRUE;
		}

		if (pMsg->wParam == VK_UP || pMsg->wParam == VK_DOWN)
		{
			ScrollHistory(pMsg->wParam == VK_DOWN);
			return TRUE;
		}
	}

	return CResizableDialog::PreTranslateMessage(pMsg);

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatWnd::Localize()
{
	static const uint16 s_auResTbl[][2] =
	{
		{ IDC_FRIENDS_LBL, IDS_CW_FRIENDS },
		{ IDC_MESSAGES_LBL, IDS_CW_MESSAGES },
		{ IDC_CSEND, IDS_CW_SEND },
		{ IDC_CCLOSE, IDS_CW_CLOSE }
	};

	if (m_hWnd)
	{
		for (unsigned ui = 0; ui < ELEMENT_COUNT(s_auResTbl); ui++)
			SetDlgItemText(s_auResTbl[ui][0], GetResString(s_auResTbl[ui][1]));
		m_FriendListCtrl.Localize();
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
LRESULT CChatWnd::OnCloseTab(WPARAM wparam, LPARAM lparam)
{
	TCITEM		tcItem;
	int			iItem = (int)wparam;

	tcItem.mask = TCIF_PARAM;
	m_ctlChatSelector.GetItem(iItem, &tcItem);
	
	m_ctlChatSelector.EndSession(((CChatItem*)tcItem.lParam)->m_pClient);

	if (iItem > 0)
		iItem--;

	if (iItem >= 0)
	{
		m_ctlChatSelector.SetCurSel(iItem);
		m_ctlChatSelector.ShowChat();
	}

	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatWnd::ScrollHistory(bool down)
{
	CChatItem		*pChatItem = m_ctlChatSelector.GetCurrentChatItem();

	if ((pChatItem == NULL) || ((pChatItem->m_iHistoryIndex == 0 && !down)
		|| (pChatItem->m_iHistoryIndex == pChatItem->m_strHistoryArray.GetCount() && down)))
		return;

	if (down)
		++pChatItem->m_iHistoryIndex;
	else
		--pChatItem->m_iHistoryIndex;

	CString			strBuffer;

	if (pChatItem->m_iHistoryIndex != pChatItem->m_strHistoryArray.GetCount())
		strBuffer = pChatItem->m_strHistoryArray.GetAt(pChatItem->m_iHistoryIndex);

	m_ctlInputText.SetWindowText(strBuffer);
	m_ctlInputText.SetSel(strBuffer.GetLength(), strBuffer.GetLength());
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatWnd::OnDestroy()
{
	CResizableDialog::OnDestroy();
	DestroyIcon(GetMessageStatic()->SetIcon(oldMessageIcon));
	DestroyIcon(GetFriendsStatic()->SetIcon(oldFriendsIcon));
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////