// PPgShortcuts.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "PPgShortcuts.h"


// CPPgShortcuts dialog

IMPLEMENT_DYNAMIC(CPPgShortcuts, CPropertyPage)
CPPgShortcuts::CPPgShortcuts()
	: CPropertyPage(CPPgShortcuts::IDD)
{
}

CPPgShortcuts::~CPPgShortcuts()
{
}

void CPPgShortcuts::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_SHORTCUTS_TREE, shortcutsTree);
	DDX_Check(pDX, IDC_SHORTCUTS_ENABLE, shortcutsEnable);
	DDX_Check(pDX, IDC_SHORTCUTS_ALT, shortcutsAlt);
	DDX_Check(pDX, IDC_SHORTCUTS_CTRL, shortcutsCtrl);
	DDX_Check(pDX, IDC_SHORTCUTS_SHIFT, shortcutsShift);
	DDX_Check(pDX, IDC_SHORTCUTS_WIN, shortcutsWin);
	DDX_Control(pDX, IDC_SHORTCUTS_ALT, shortcutsAltCheck);
	DDX_Control(pDX, IDC_SHORTCUTS_CTRL, shortcutsCtrlCheck);
	DDX_Control(pDX, IDC_SHORTCUTS_SHIFT, shortcutsShiftCheck);
	DDX_Control(pDX, IDC_SHORTCUTS_WIN, shortcutsWinCheck);
	DDX_Control(pDX, IDC_SHORTCUTS_STATIC, shortcutsStaticLbl);
	DDX_Control(pDX, IDC_SHORTCUTS_COMBO, shortcutsKeyCombo);
	DDX_Control(pDX, IDC_SHORTCUTS_GROUPBOX, shortcutsGroupBox);
}


BEGIN_MESSAGE_MAP(CPPgShortcuts, CPropertyPage)
	ON_BN_CLICKED(IDC_SHORTCUTS_ENABLE, /*OnSettingsChange*/OnEnableShortcutClick)
	ON_BN_CLICKED(IDC_SHORTCUTS_ALT, OnSettingsChange)
	ON_BN_CLICKED(IDC_SHORTCUTS_CTRL, OnSettingsChange)
	ON_BN_CLICKED(IDC_SHORTCUTS_SHIFT, OnSettingsChange)
	ON_BN_CLICKED(IDC_SHORTCUTS_WIN, OnSettingsChange)
	ON_CBN_SELCHANGE(IDC_SHORTCUTS_COMBO, OnSettingsChange)
	ON_NOTIFY(TVN_SELCHANGING, IDC_SHORTCUTS_TREE, OnTvnSelchangingShortcutsTree)
	ON_WM_DESTROY()
END_MESSAGE_MAP()


// Gestionnaires de messages CPPgShortcuts

BOOL CPPgShortcuts::OnInitDialog()
{
	CPropertyPage::OnInitDialog();

	CreateShortcutsTree();	// WARNING : shortcuts tree
	RefreshKeyCombo();		//           and key combo must have been created
	LoadSettings();			//           before calling LoadSettings function!
	Localize();

	return TRUE;
}

void CPPgShortcuts::OnDestroy()
{
//	Free resources on close not to keep them until next creation of this page
	imagelistShortcutsTree.DeleteImageList();
	shortcutsTree.DeleteAllItems();

	CPropertyPage::OnDestroy();
}

BOOL CPPgShortcuts::OnApply()
{
	for (int i = 0; i < NUMBER_OF_ACTIONS; i++)
		m_pPrefs->SetShortcutCode(m_ShortcutTreeItem[i].Shortcut.GetCode(), i);

	RefreshShortcutsTree();

	return CPropertyPage::OnApply();
}

void CPPgShortcuts::OnSettingsChange(void)
{
	SetModified();
	SaveShortcut(shortcutsTree.GetSelectedItem());
}

void CPPgShortcuts::OnEnableShortcutClick(void)
{
	OnSettingsChange();
	UpdateCheckBoxesState();
}

void CPPgShortcuts::CreateShortcutsTree(void)
{
	static const uint16 s_auIconResID[] =
	{
	//	Generic icon
		IDI_TVI_GENERIC,	// 0 - items
	//	Main category icons
		IDI_TRAYICON,		// 1 - General
		IDI_DIRECTDOWNLOAD,	// 2 - Download list
		IDI_UPLOAD,			// 3 - Upload list
		IDI_PREF_FILES,		// 4 - Share list
	//	Sub-category icons
		IDI_PREF_WINDOW,	// 5 - General > Show window
		IDI_PREF_ADVANCED,	// 6 - General > Adanved
		IDI_GLOBAL			// 7 - UL & Share > ED2K Links
	};

	imagelistShortcutsTree.Create(16, 16, ILC_COLOR32 | ILC_MASK, ELEMENT_COUNT(s_auIconResID), 0);
	FillImgLstWith16x16Icons(&imagelistShortcutsTree, s_auIconResID, ELEMENT_COUNT(s_auIconResID));
	shortcutsTree.SetImageList(&imagelistShortcutsTree, TVSIL_NORMAL);

	h_general = shortcutsTree.InsertItem(GetResString(IDS_PW_GENERAL), 1, 1);
	h_download = shortcutsTree.InsertItem(GetResString(IDS_TREE_DL), 2, 2);
	h_upload = shortcutsTree.InsertItem(GetResString(IDS_TREE_UL), 3, 3);
	h_share = shortcutsTree.InsertItem(GetResString(IDS_TREE_SHARE), 4, 4);
	m_ShortcutTreeItem[IndexGalMinimize].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexGalMinimize].uStringID		= IDS_TREE_GENERAL_MINIMIZE),			h_general);
	m_ShortcutTreeItem[IndexGalSwitchWindow].h_Handle	= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexGalSwitchWindow].uStringID	= IDS_TREE_GENERAL_SWITCH_WND),			h_general);
	h_generalShowWindow = shortcutsTree.InsertItem(GetResString(IDS_TREE_GENERAL_SHOW_WND), 5, 5, h_general);
	m_ShortcutTreeItem[IndexGalShowServer].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexGalShowServer].uStringID	= IDS_TREE_GENERAL_SHOW_SERVER),		h_generalShowWindow);
	m_ShortcutTreeItem[IndexGalShowTransfer].h_Handle	= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexGalShowTransfer].uStringID	= IDS_TREE_GENERAL_SHOW_TRANSFER),		h_generalShowWindow);
	m_ShortcutTreeItem[IndexGalShowSearch].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexGalShowSearch].uStringID	= IDS_TREE_GENERAL_SHOW_SEARCH),		h_generalShowWindow);
	m_ShortcutTreeItem[IndexGalShowShare].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexGalShowShare].uStringID		= IDS_TREE_GENERAL_SHOW_SHAREDFILES),	h_generalShowWindow);
	m_ShortcutTreeItem[IndexGalShowChat].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexGalShowChat].uStringID		= IDS_TREE_GENERAL_SHOW_CHAT),			h_generalShowWindow);
	m_ShortcutTreeItem[IndexGalShowIRC].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexGalShowIRC].uStringID		= IDS_TREE_GENERAL_SHOW_IRC),			h_generalShowWindow);
	m_ShortcutTreeItem[IndexGalShowStats].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexGalShowStats].uStringID		= IDS_TREE_GENERAL_SHOW_STATISTICS),	h_generalShowWindow);
	m_ShortcutTreeItem[IndexGalShowPrefs].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexGalShowPrefs].uStringID		= IDS_TREE_GENERAL_SHOW_PREFERENCES),	h_generalShowWindow);
	m_ShortcutTreeItem[IndexDLCancel].h_Handle			= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLCancel].uStringID			= IDS_TREE_DL_CANCEL),					h_download);
	m_ShortcutTreeItem[IndexDLStop].h_Handle			= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLStop].uStringID			= IDS_TREE_DL_STOP),					h_download);
	m_ShortcutTreeItem[IndexDLPause].h_Handle			= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLPause].uStringID			= IDS_TREE_DL_PAUSE),					h_download);
	m_ShortcutTreeItem[IndexDLResume].h_Handle			= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLResume].uStringID			= IDS_TREE_DL_RESUME),					h_download);
	m_ShortcutTreeItem[IndexDLOpen].h_Handle			= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLOpen].uStringID			= IDS_TREE_DL_OPEN),					h_download);
	m_ShortcutTreeItem[IndexDLOpenFolder].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLOpenFolder].uStringID		= IDS_TREE_SHARE_OPENFOLDER),			h_download);
	m_ShortcutTreeItem[IndexDLPreview].h_Handle			= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLPreview].uStringID		= IDS_TREE_DL_PREVIEW),					h_download);
	m_ShortcutTreeItem[IndexDLRename].h_Handle			= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLRename].uStringID			= IDS_TREE_DL_RENAME),					h_download);
	m_ShortcutTreeItem[IndexDLComments].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLComments].uStringID		= IDS_TREE_DL_VIEW_COMMENTS),			h_download);
	m_ShortcutTreeItem[IndexDLDetails].h_Handle			= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLDetails].uStringID		= IDS_TREE_DL_VIEW_DETAILS),			h_download);
	m_ShortcutTreeItem[IndexDLDetailsSources].h_Handle	= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLDetailsSources].uStringID	= IDS_TREE_DL_VIEW_DETAILSSOURCES),		h_download);
	m_ShortcutTreeItem[IndexDLClear].h_Handle			= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLClear].uStringID			= IDS_TREE_DL_CLEAR_COMPLETED),			h_download);
	m_ShortcutTreeItem[IndexDLClearAll].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLClearAll].uStringID		= IDS_TREE_DL_CLEAR_ALL_COMPLETED),		h_download);
	m_ShortcutTreeItem[IndexDLShowAllUp].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLShowAllUp].uStringID		= IDS_TREE_DL_SHOW_ALLUP),				h_download);
	m_ShortcutTreeItem[IndexDLUseDefaultSort].h_Handle	= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLUseDefaultSort].uStringID	= IDS_TREE_DL_USE_SORT),				h_download);
	h_downloadAdvanced = shortcutsTree.InsertItem(GetResString(IDS_PW_ADVANCED), 6, 6, h_download);
	m_ShortcutTreeItem[IndexDLPreallocate].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLPreallocate].uStringID	= IDS_TREE_DL_PREALLOCATE),				h_downloadAdvanced);
	m_ShortcutTreeItem[IndexDLA4AF].h_Handle			= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLA4AF].uStringID			= IDS_TREE_DL_A4AF),					h_downloadAdvanced);
	m_ShortcutTreeItem[IndexDLA4AFAuto].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLA4AFAuto].uStringID		= IDS_TREE_DL_A4AF_AUTO),				h_downloadAdvanced);
	m_ShortcutTreeItem[IndexDLA4AFOther].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLA4AFOther].uStringID		= IDS_ALL_A4AF_TO_OTHER),				h_downloadAdvanced);
	h_downloadED2KLinks = shortcutsTree.InsertItem(GetResString(IDS_ED2KLINKFIX), 7, 7, h_download);
	m_ShortcutTreeItem[IndexDLED2KLink].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLED2KLink].uStringID		= IDS_TREE_DL_ED2KLINK),				h_downloadED2KLinks);
	m_ShortcutTreeItem[IndexDLED2kLinkHtml].h_Handle	= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLED2kLinkHtml].uStringID	= IDS_TREE_DL_ED2KLINK_HTML),			h_downloadED2KLinks);
	m_ShortcutTreeItem[IndexDLED2kLinkSource].h_Handle	= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexDLED2kLinkSource].uStringID	= IDS_TREE_DL_ED2KLINK_SOURCE),			h_downloadED2KLinks);
	m_ShortcutTreeItem[IndexULDetails].h_Handle			= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexULDetails].uStringID		= IDS_TREE_UL_VIEW_DETAILS),			h_upload);
	m_ShortcutTreeItem[IndexULFriend].h_Handle			= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexULFriend].uStringID			= IDS_TREE_UL_ADD_FRIEND),				h_upload);
	m_ShortcutTreeItem[IndexULMessage].h_Handle			= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexULMessage].uStringID		= IDS_TREE_UL_SEND_MESSAGE),			h_upload);
	m_ShortcutTreeItem[IndexULFiles].h_Handle			= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexULFiles].uStringID			= IDS_TREE_UL_VIEW_FILES),				h_upload);
	m_ShortcutTreeItem[IndexShOpenFolder].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexShOpenFolder].uStringID		= IDS_TREE_SHARE_OPENFOLDER),			h_share);
	m_ShortcutTreeItem[IndexShComments].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexShComments].uStringID		= IDS_TREE_SHARE_EDIT_COMMENTS),		h_share);
	h_shareED2KLinks = shortcutsTree.InsertItem(GetResString(IDS_ED2KLINKFIX), 7, 7, h_share);
	m_ShortcutTreeItem[IndexShED2KLink].h_Handle		= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexShED2KLink].uStringID		= IDS_TREE_DL_ED2KLINK),				h_shareED2KLinks);
	m_ShortcutTreeItem[IndexShED2kLinkHtml].h_Handle	= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexShED2kLinkHtml].uStringID	= IDS_TREE_DL_ED2KLINK_HTML),			h_shareED2KLinks);
	m_ShortcutTreeItem[IndexShED2kLinkSource].h_Handle	= shortcutsTree.InsertItem(GetResString(m_ShortcutTreeItem[IndexShED2kLinkSource].uStringID	= IDS_TREE_DL_ED2KLINK_SOURCE),			h_shareED2KLinks);

	shortcutsTree.SetItemState(h_general, TVIS_BOLD, TVIS_BOLD);
	shortcutsTree.SetItemState(h_generalShowWindow, TVIS_BOLD, TVIS_BOLD);
	shortcutsTree.SetItemState(h_download, TVIS_BOLD, TVIS_BOLD);
	shortcutsTree.SetItemState(h_downloadAdvanced, TVIS_BOLD, TVIS_BOLD);
	shortcutsTree.SetItemState(h_downloadED2KLinks, TVIS_BOLD, TVIS_BOLD);
	shortcutsTree.SetItemState(h_upload, TVIS_BOLD, TVIS_BOLD);
	shortcutsTree.SetItemState(h_share, TVIS_BOLD, TVIS_BOLD);
	shortcutsTree.SetItemState(h_shareED2KLinks, TVIS_BOLD, TVIS_BOLD);

	shortcutsTree.Expand(h_general, TVE_EXPAND);
	shortcutsTree.Expand(h_generalShowWindow, TVE_EXPAND);
	shortcutsTree.Expand(h_downloadAdvanced, TVE_EXPAND);
	shortcutsTree.Expand(h_downloadED2KLinks, TVE_EXPAND);
	shortcutsTree.Expand(h_shareED2KLinks, TVE_EXPAND);

	shortcutsTree.Select(m_ShortcutTreeItem[0].h_Handle, TVGN_CARET);
}

void CPPgShortcuts::RefreshShortcutsTree(void)
{
	shortcutsTree.SetItemText(h_general, GetResString(IDS_PW_GENERAL));
	shortcutsTree.SetItemText(h_generalShowWindow, GetResString(IDS_TREE_GENERAL_SHOW_WND));
	shortcutsTree.SetItemText(h_download, GetResString(IDS_TREE_DL));
	shortcutsTree.SetItemText(h_downloadAdvanced, GetResString(IDS_PW_ADVANCED));
	shortcutsTree.SetItemText(h_downloadED2KLinks, GetResString(IDS_ED2KLINKFIX));
	shortcutsTree.SetItemText(h_upload, GetResString(IDS_TREE_UL));
	shortcutsTree.SetItemText(h_share, GetResString(IDS_TREE_SHARE));
	shortcutsTree.SetItemText(h_shareED2KLinks, GetResString(IDS_ED2KLINKFIX));

	for (int i = 0; i < NUMBER_OF_ACTIONS; i++)
		shortcutsTree.SetItemText(m_ShortcutTreeItem[i].h_Handle, GetResString(m_ShortcutTreeItem[i].uStringID) + GetStringFromShortcutCode(i, SSP_SPACE_PREFIX, TRUE));
}

void CPPgShortcuts::RefreshKeyCombo(void)
{
	shortcutsKeyCombo.ResetContent();
	shortcutsKeyCombo.AddString(GetResString(IDS_SHORTCUTS_ENTER_KEY));	//  0
	shortcutsKeyCombo.AddString(GetResString(IDS_SHORTCUTS_SPACE_KEY));	//  1
	shortcutsKeyCombo.AddString(GetResString(IDS_SHORTCUTS_TAB_KEY));	//  2
	shortcutsKeyCombo.AddString(GetResString(IDS_SHORTCUTS_BACK_KEY));	//  3
	shortcutsKeyCombo.AddString(GetResString(IDS_SHORTCUTS_INSERT_KEY));//  4
	shortcutsKeyCombo.AddString(GetResString(IDS_SHORTCUTS_DELETE_KEY));//  5
	shortcutsKeyCombo.AddString(GetResString(IDS_SHORTCUTS_HOME_KEY));  //  6
	shortcutsKeyCombo.AddString(GetResString(IDS_SHORTCUTS_END_KEY));	//  7
	shortcutsKeyCombo.AddString(GetResString(IDS_SHORTCUTS_PAGEUP_KEY));//  8
	shortcutsKeyCombo.AddString(GetResString(IDS_SHORTCUTS_PAGEDOWN_KEY));//9
	shortcutsKeyCombo.AddString("F1");		// 10
	shortcutsKeyCombo.AddString("F2");		// 11
	shortcutsKeyCombo.AddString("F3");		// 12
	shortcutsKeyCombo.AddString("F4");		// 13
	shortcutsKeyCombo.AddString("F5");		// 14
	shortcutsKeyCombo.AddString("F6");		// 15
	shortcutsKeyCombo.AddString("F7");		// 16
	shortcutsKeyCombo.AddString("F8");		// 17
	shortcutsKeyCombo.AddString("F9");		// 18
	shortcutsKeyCombo.AddString("F10");		// 19
	shortcutsKeyCombo.AddString("F11");		// 20
	shortcutsKeyCombo.AddString("F12");		// 21
	shortcutsKeyCombo.AddString("A");		// 22 - letters start
	shortcutsKeyCombo.AddString("B");		// 23
	shortcutsKeyCombo.AddString("C");		// 24
	shortcutsKeyCombo.AddString("D");		// 25
	shortcutsKeyCombo.AddString("E");		// 26
	shortcutsKeyCombo.AddString("F");		// 27
	shortcutsKeyCombo.AddString("G");		// 28
	shortcutsKeyCombo.AddString("H");		// 29
	shortcutsKeyCombo.AddString("I");		// 30
	shortcutsKeyCombo.AddString("J");		// 31
	shortcutsKeyCombo.AddString("K");		// 32
	shortcutsKeyCombo.AddString("L");		// 33
	shortcutsKeyCombo.AddString("M");		// 34
	shortcutsKeyCombo.AddString("N");		// 35
	shortcutsKeyCombo.AddString("O");		// 36
	shortcutsKeyCombo.AddString("P");		// 37
	shortcutsKeyCombo.AddString("Q");		// 38
	shortcutsKeyCombo.AddString("R");		// 39
	shortcutsKeyCombo.AddString("S");		// 40
	shortcutsKeyCombo.AddString("T");		// 41
	shortcutsKeyCombo.AddString("U");		// 42
	shortcutsKeyCombo.AddString("V");		// 43
	shortcutsKeyCombo.AddString("W");		// 44
	shortcutsKeyCombo.AddString("X");		// 45
	shortcutsKeyCombo.AddString("Y");		// 46
	shortcutsKeyCombo.AddString("Z");		// 47 - letters end
	shortcutsKeyCombo.AddString("0");		// 48 - numbers start
	shortcutsKeyCombo.AddString("1");		// 49
	shortcutsKeyCombo.AddString("2");		// 50
	shortcutsKeyCombo.AddString("3");		// 51
	shortcutsKeyCombo.AddString("4");		// 52
	shortcutsKeyCombo.AddString("5");		// 53
	shortcutsKeyCombo.AddString("6");		// 54
	shortcutsKeyCombo.AddString("7");		// 55
	shortcutsKeyCombo.AddString("8");		// 56
	shortcutsKeyCombo.AddString("9");		// 57 - numbers end
	int iIndex = GetShortcutTreeItemIndexFromItem(shortcutsTree.GetSelectedItem());
	if (iIndex >= 0)
		shortcutsKeyCombo.SetCurSel(GetKeyComboIndexFromShortcutKey(m_ShortcutTreeItem[iIndex].Shortcut.GetKey()));
}

void CPPgShortcuts::LoadSettings(void)
{
	for (int i = 0; i < NUMBER_OF_ACTIONS; i++)
		m_ShortcutTreeItem[i].Shortcut.SetCode(m_pPrefs->GetShortcutCode(i));

	LoadShortcut(shortcutsTree.GetSelectedItem());

	SetModified(FALSE);
}

void CPPgShortcuts::Localize(void)
{
	if(::IsWindow(m_hWnd))
	{
		SetWindowText(GetResString(IDS_PW_SHORTCUTS));
		shortcutsGroupBox.SetWindowText("      " + GetResString(IDS_SHORTCUTS_ENABLE)); // the spaces let a blank space to display the IDC_SHORTCUTS_ENABLE checkbox
		shortcutsStaticLbl.SetWindowText(GetResString(IDS_SHORTCUTS_STATIC));
		RefreshShortcutsTree();
		RefreshKeyCombo();
	}
}

int CPPgShortcuts::GetShortcutTreeItemIndexFromItem(HTREEITEM hItem)
{
	for (int i = 0; i < NUMBER_OF_ACTIONS; i++)
		if (m_ShortcutTreeItem[i].h_Handle == hItem)
			return i;

	return -1;
}

void CPPgShortcuts::LoadShortcut(HTREEITEM hItem)
{
	int i = GetShortcutTreeItemIndexFromItem(hItem);

	if (i >= 0)
	{
		shortcutsEnable	= m_ShortcutTreeItem[i].Shortcut.IsEnabled();
		shortcutsAlt	= m_ShortcutTreeItem[i].Shortcut.IsAlt();
		shortcutsCtrl	= m_ShortcutTreeItem[i].Shortcut.IsCtrl();
		shortcutsShift	= m_ShortcutTreeItem[i].Shortcut.IsShift();
		shortcutsWin	= m_ShortcutTreeItem[i].Shortcut.IsWin();
		shortcutsKeyCombo.SetCurSel(GetKeyComboIndexFromShortcutKey(m_ShortcutTreeItem[i].Shortcut.GetKey()));
		UpdateData(FALSE);
		UpdateCheckBoxesState();
	}
}

void CPPgShortcuts::SaveShortcut(HTREEITEM hItem)
{
	int i = GetShortcutTreeItemIndexFromItem(hItem);

	if (i >= 0)
	{
		UpdateData();
		m_ShortcutTreeItem[i].Shortcut.SetEnabled(shortcutsEnable);
		m_ShortcutTreeItem[i].Shortcut.SetAlt(shortcutsAlt);
		m_ShortcutTreeItem[i].Shortcut.SetCtrl(shortcutsCtrl);
		m_ShortcutTreeItem[i].Shortcut.SetShift(shortcutsShift);
		m_ShortcutTreeItem[i].Shortcut.SetWin(shortcutsWin);
		m_ShortcutTreeItem[i].Shortcut.SetKey(GetShortcutKeyFromKeyComboIndex(shortcutsKeyCombo.GetCurSel()));
	}
}

int CPPgShortcuts::GetKeyComboIndexFromShortcutKey(char cKey) const
{
	if (cKey == VK_RETURN) // Enter key
		return 0;
	else if (cKey == VK_SPACE) // Spacebar
		return 1;
	else if (cKey == VK_TAB) // Tab key
		return 2;
	else if (cKey == VK_BACK) // Backspace key
		return 3;
	else if (cKey == VK_INSERT) // Insert key
		return 4;
	else if (cKey == VK_DELETE) // Delete key
		return 5;
	else if (cKey == VK_HOME) // Home key
		return 6;
	else if (cKey == VK_END) // End key
		return 7;
	else if (cKey == VK_PRIOR) // Page Up key
		return 8;
	else if (cKey == VK_NEXT) // Page Down key
		return 9;
	else if ((cKey >= VK_F1) && (cKey <= VK_F12)) // function keys
		return (cKey - VK_F1) + 10;
	else if ((cKey >= 'A') && (cKey <= 'Z')) // letter keys
		return (cKey - 'A') + 22;
	else if ((cKey >= '0') && (cKey <= '9')) // number keys
		return (cKey - '0') + 48;
	else /*** should never happen ***/
	{
		AfxMessageBox("Error in key code. 'Enter' key was set to default.", MB_OK | MB_ICONERROR);
		return 0; // return Enter key index by default
	}
}

char CPPgShortcuts::GetShortcutKeyFromKeyComboIndex(int iIndex) const
{
	if (iIndex == 0) // Enter key
		return VK_RETURN;
	else if (iIndex == 1) // Spacebar
		return VK_SPACE;
	else if (iIndex == 2) // Tab key
		return VK_TAB;
	else if (iIndex == 3) // Backspace key
		return VK_BACK;
	else if (iIndex == 4) // Insert key
		return VK_INSERT;
	else if (iIndex == 5) // Delete key
		return VK_DELETE;
	else if (iIndex == 6) // Home key
		return VK_HOME;
	else if (iIndex == 7) // End key
		return VK_END;
	else if (iIndex == 8) // Page Up key
		return VK_PRIOR;
	else if (iIndex == 9) // Page Down key
		return VK_NEXT;
	else if ((iIndex >= 10) && (iIndex <= 21)) // function keys
		return VK_F1 + (iIndex - 10);
	else if ((iIndex >= 22) && (iIndex <= 47)) // letter keys
		return 'A' + (iIndex - 22);
	else if ((iIndex >= 48) && (iIndex <= 57)) // number keys
		return '0' + (iIndex - 48);
	else /*** should never happen ***/
	{
		AfxMessageBox("Error in key combo index. 'Enter' key was set to default.", MB_OK | MB_ICONERROR);
		return VK_RETURN;
	}
}

void CPPgShortcuts::UpdateCheckBoxesState(void)
{
	UpdateData();
	shortcutsAltCheck.EnableWindow(shortcutsEnable);
	shortcutsCtrlCheck.EnableWindow(shortcutsEnable);
	shortcutsShiftCheck.EnableWindow(shortcutsEnable);
	shortcutsWinCheck.EnableWindow(shortcutsEnable);
	shortcutsStaticLbl.EnableWindow(shortcutsEnable);
	shortcutsKeyCombo.EnableWindow(shortcutsEnable);
}

// This function will prevent the user to select root items
void CPPgShortcuts::OnTvnSelchangingShortcutsTree(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);
	
	if (shortcutsTree.ItemHasChildren(pNMTreeView->itemNew.hItem))
		*pResult = TRUE;
	else
	{
		LoadShortcut(pNMTreeView->itemNew.hItem);
		*pResult = FALSE;
	}
}
